@@ -54,10 +54,12 @@ void AudioOutput::handleConnectToMediaObject(MediaObject *mediaObject)
 void AudioOutput::handleAddToMedia(Media *media)
 {
     media->addOption(":audio");
+#if (PHONON_VERSION >= PHONON_VERSION_CHECK(4, 6, 50))
     PulseSupport *pulse = PulseSupport::getInstance();
     if (pulse && pulse->isActive()) {
         pulse->setupStreamEnvironment(m_streamUuid);
     }
+#endif
 }
 
 qreal AudioOutput::volume() const
@@ -115,6 +117,8 @@ bool AudioOutput::setOutputDevice(const AudioOutputDevice &newDevice)
 #if (PHONON_VERSION >= PHONON_VERSION_CHECK(4, 6, 50))
 void AudioOutput::setStreamUuid(QString uuid)
 {
+    DEBUG_BLOCK;
+    debug() << uuid;
     m_streamUuid = uuid;
 }
 #endif
