@@ -3,7 +3,7 @@
     Copyright (C) 2008 Lukas Durfina <lukas.durfina@gmail.com>
     Copyright (C) 2009 Fathi Boudra <fabo@kde.org>
     Copyright (C) 2010 Ben Cooksley <sourtooth@gmail.com>
-    Copyright (C) 2009-2011 vlc-phonon AUTHORS
+    Copyright (C) 2009-2011 vlc-phonon AUTHORS <kde-multimedia@kde.org>
     Copyright (C) 2010-2011 Harald Sitter <sitter@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -22,9 +22,9 @@
 
 #include "mediaobject.h"
 
+#include <QtCore/QDir>
 #include <QtCore/QStringBuilder>
 #include <QtCore/QUrl>
-#include <QDir>
 
 #include <vlc/libvlc_version.h>
 #include <vlc/vlc.h>
@@ -67,6 +67,7 @@ MediaObject::MediaObject(QObject *parent)
 
     // Internal Signals.
     connect(this, SIGNAL(moveToNext()), SLOT(moveToNextSource()));
+    connect(m_refreshTimer, SIGNAL(timeout()), this, SLOT(refreshDescriptors()));
 
     resetMembers();
 }
@@ -339,10 +340,9 @@ void MediaObject::setSource(const MediaSource &source)
         case Phonon::Vcd:
             loadMedia(QLatin1Literal("vcd://") % m_mediaSource.deviceName());
             break;
-#if (PHONON_VERSION >= PHONON_VERSION_CHECK(4, 7, 0))
         case Phonon::BluRay:
             loadMedia(QLatin1Literal("bluray://") % m_mediaSource.deviceName());
-#endif
+            break;
         }
         break;
     case MediaSource::CaptureDevice: {
@@ -413,6 +413,14 @@ void MediaObject::setNextSource(const MediaSource &source)
     DEBUG_BLOCK;
     debug() << source.url();
     m_nextSource = source;
+    // This function is not ever called by the consumer but only libphonon.
+    // Furthermore libphonon only calls this function in its aboutToFinish slot,
+    // iff sources are already in the queue. In case our aboutToFinish was too
+    // late we may already be stopped when the slot gets activated.
+    // Therefore we need to make sure that we move to the next source iff
+    // this function is called when we are in stoppedstate.
+    if (m_state == StoppedState)
+        moveToNext();
 }
 
 qint32 MediaObject::prefinishMark() const
@@ -452,14 +460,10 @@ void MediaObject::emitAboutToFinish()
 void MediaObject::changeState(Phonon::State newState)
 {
     DEBUG_BLOCK;
-    if (newState == m_state) {
-        // State not changed
+
+    // State not changed
+    if (newState == m_state)
         return;
-    } else if (checkGaplessWaiting()) {
-        // This is a no-op, warn that we are....
-        debug() << Q_FUNC_INFO << "no-op gapless item awaiting in queue - " << m_nextSource.type() ;
-        return;
-    }
 
     debug() << m_state << "-->" << newState;
 
@@ -484,19 +488,14 @@ void MediaObject::changeState(Phonon::State newState)
 void MediaObject::moveToNextSource()
 {
     DEBUG_BLOCK;
-    if (m_nextSource.type() == MediaSource::Invalid) {
-        // No item is scheduled to be next...
-        return;
-    }
 
     setSource(m_nextSource);
     play();
     m_nextSource = MediaSource(QUrl());
 }
 
-inline bool MediaObject::checkGaplessWaiting()
+inline bool MediaObject::hasNextTrack()
 {
-    DEBUG_BLOCK;
     return m_nextSource.type() != MediaSource::Invalid && m_nextSource.type() != MediaSource::Empty;
 }
 
@@ -526,10 +525,8 @@ void MediaObject::setupMedia()
         m_media->addOption(QLatin1String("screen-caching=300"));
     }
 
-    if (source().discType() == Cd && m_currentTitle > 0) {
-        debug() << "setting CDDA track";
-        m_media->addOption(QLatin1String("cdda-track="), QVariant(m_currentTitle));
-    }
+    if (source().discType() == Cd && m_currentTitle > 0)
+        m_media->setCdTrack(m_currentTitle);
 
     if (m_streamReader)
         // StreamReader is no sink but a source, for this we have no concept right now
@@ -537,6 +534,28 @@ void MediaObject::setupMedia()
         // Consequently we need to manually tell the StreamReader to attach to the Media.
         m_streamReader->addToMedia(m_media);
 
+    if (!m_subtitleAutodetect)
+        m_media->addOption(QLatin1String(":no-sub-autodetect-file"));
+
+    if (m_subtitleEncoding != QLatin1String("UTF-8")) // utf8 is phonon default, so let vlc handle it
+        m_media->addOption(QLatin1String(":subsdec-encoding="), m_subtitleEncoding);
+
+    if (!m_subtitleFontChanged) // Update font settings
+        m_subtitleFont = QFont();
+
+#ifdef __GNUC__
+#warning freetype module is not working as expected - font api not working
+#endif
+    // BUG: VLC's freetype module doesn't pick up per-media options
+    // vlc -vvvv --freetype-font="Comic Sans MS" multiple_sub_sample.mkv :freetype-font=Arial
+    // https://trac.videolan.org/vlc/ticket/9797
+    m_media->addOption(QLatin1String(":freetype-font="), m_subtitleFont.family());
+    m_media->addOption(QLatin1String(":freetype-fontsize="), m_subtitleFont.pointSize());
+    if (m_subtitleFont.bold())
+        m_media->addOption(QLatin1String(":freetype-bold"));
+    else
+        m_media->addOption(QLatin1String(":no-freetype-bold"));
+
     foreach (SinkNode *sink, m_sinks) {
         sink->addToMedia(m_media);
     }
@@ -580,7 +599,7 @@ void MediaObject::updateDuration(qint64 newDuration)
     // This here cache is needed because we need to provide -1 as totalTime()
     // for as long as we do not get a proper update through this slot.
     // VLC reports -1 with no media but 0 if it does not know the duration, so
-    // apps that assume 0 = unkown get screwed if they query too early.
+    // apps that assume 0 = unknown get screwed if they query too early.
     // http://bugs.tomahawk-player.org/browse/TWK-1029
     m_totalTime = newDuration;
     emit totalTimeChanged(m_totalTime);
@@ -626,6 +645,27 @@ void MediaObject::updateState(MediaPlayer::State state)
 {
     DEBUG_BLOCK;
     debug() << state;
+    debug() << "attempted autoplay?" << m_attemptingAutoplay;
+
+    if (m_attemptingAutoplay) {
+        switch (state) {
+        case MediaPlayer::PlayingState:
+        case MediaPlayer::PausedState:
+            m_attemptingAutoplay = false;
+            break;
+        case MediaPlayer::ErrorState:
+            debug() << "autoplay failed, must be end of media.";
+            // The error should not be reflected to the consumer. So we swap it
+            // for finished() which is actually what is happening here.
+            // Or so we think ;)
+            state = MediaPlayer::EndedState;
+            --m_currentTitle;
+            break;
+        default:
+            debug() << "not handling as part of autplay:" << state;
+            break;
+        }
+    }
 
     switch (state) {
     case MediaPlayer::NoState:
@@ -647,9 +687,18 @@ void MediaObject::updateState(MediaPlayer::State state)
         changeState(StoppedState);
         break;
     case MediaPlayer::EndedState:
-        emitAboutToFinish();
-        emit finished();
-        changeState(StoppedState);
+        if (hasNextTrack()) {
+            moveToNextSource();
+        } else if (source().discType() == Cd && m_autoPlayTitles && !m_attemptingAutoplay) {
+            debug() << "trying to simulate autoplay";
+            m_attemptingAutoplay = true;
+            m_player->setCdTrack(++m_currentTitle);
+        } else {
+            m_attemptingAutoplay = false;
+            emitAboutToFinish();
+            emit finished();
+            changeState(StoppedState);
+        }
         break;
     case MediaPlayer::ErrorState:
         debug() << errorString();
@@ -695,24 +744,14 @@ void MediaObject::onHasVideoChanged(bool hasVideo)
         // MC and MO are force-reset on media changes anyway.
         return;
 
-    if (hasVideo) {
-        refreshAudioChannels();
-        refreshSubtitles();
-
-        // Get movie chapter count
-        // It is not a title/chapter media if there is no chapter
-        if (m_player->videoChapterCount() > 0) {
-            refreshTitles();
-            refreshChapters(m_player->title());
-        }
-    }
+    refreshDescriptors();
 }
 
 void MediaObject::setBufferStatus(int percent)
 {
     // VLC does not have a buffering state (surprise!) but instead only sends the
     // event (surprise!). Hence we need to simulate the state change.
-    // Problem with BufferingState is that it is actually concurent to Playing or Paused
+    // Problem with BufferingState is that it is actually concurrent to Playing or Paused
     // meaning that while you are buffering you can also pause, thus triggering
     // a state change to paused. To handle this we let updateState change the
     // state accordingly (as we need to allow the UI to update itself, and
@@ -736,6 +775,20 @@ void MediaObject::setBufferStatus(int percent)
     }
 }
 
+void MediaObject::refreshDescriptors()
+{
+    if (m_player->titleCount() > 0)
+        refreshTitles();
+
+    if (hasVideo()) {
+        refreshAudioChannels();
+        refreshSubtitles();
+
+        if (m_player->videoChapterCount() > 0)
+            refreshChapters(m_player->title());
+    }
+}
+
 qint64 MediaObject::totalTime() const
 {
     return m_totalTime;
