@@ -47,9 +47,12 @@
 namespace Phonon {
 namespace VLC {
 
-MediaPlayer::MediaPlayer(QObject *parent) :
-    QObject(parent),
-    m_player(libvlc_media_player_new(libvlc))
+MediaPlayer::MediaPlayer(QObject *parent)
+    : QObject(parent)
+    , m_player(libvlc_media_player_new(libvlc))
+    , m_doingPausedPlay(false)
+    , m_volume(75)
+    , m_fadeAmount(1.0f)
 {
     Q_ASSERT(m_player);
 
@@ -81,6 +84,13 @@ MediaPlayer::MediaPlayer(QObject *parent) :
     for (int i = 0; i < eventCount; ++i) {
         libvlc_event_attach(manager, events[i], event_cb, this);
     }
+
+    // Deactivate video title overlay (i.e. name of the video displaying
+    // at start. Since 2.1 that is handled via the API which in general is more
+    // reliable than setting it via libvlc_new (or so I have been told....)
+#if (LIBVLC_VERSION_INT >= LIBVLC_VERSION(2, 1, 0, 0))
+    libvlc_media_player_set_video_title_display(m_player, libvlc_position_disable, 0);
+#endif
 }
 
 MediaPlayer::~MediaPlayer()
@@ -96,16 +106,25 @@ void MediaPlayer::setMedia(Media *media)
 
 bool MediaPlayer::play()
 {
+    m_doingPausedPlay = false;
     return libvlc_media_player_play(m_player) == 0;
 }
 
 void MediaPlayer::pause()
 {
+    m_doingPausedPlay = false;
     libvlc_media_player_set_pause(m_player, 1);
 }
 
+void MediaPlayer::pausedPlay()
+{
+    m_doingPausedPlay = true;
+    libvlc_media_player_play(m_player);
+}
+
 void MediaPlayer::resume()
 {
+    m_doingPausedPlay = false;
     libvlc_media_player_set_pause(m_player, 0);
 }
 
@@ -116,6 +135,7 @@ void MediaPlayer::togglePause()
 
 void MediaPlayer::stop()
 {
+    m_doingPausedPlay = false;
     libvlc_media_player_stop(m_player);
 }
 
@@ -151,7 +171,7 @@ bool MediaPlayer::setSubtitle(int subtitle)
 
 bool MediaPlayer::setSubtitle(const QString &file)
 {
-    return libvlc_video_set_subtitle_file(m_player, file.toAscii().data()) == 0;
+    return libvlc_video_set_subtitle_file(m_player, file.toUtf8().data()) == 1;
 }
 
 void MediaPlayer::setTitle(int title)
@@ -219,7 +239,12 @@ void MediaPlayer::event_cb(const libvlc_event_t *event, void *opaque)
                     Q_ARG(int, event->u.media_player_buffering.new_cache));
         break;
     case libvlc_MediaPlayerPlaying:
-        P_EMIT_STATE(PlayingState);
+        // Intercept state change and apply pausing once playing.
+        if (that->m_doingPausedPlay) {
+            that->m_doingPausedPlay = false;
+            that->pause();
+        } else
+            P_EMIT_STATE(PlayingState);
         break;
     case libvlc_MediaPlayerPaused:
         P_EMIT_STATE(PausedState);
@@ -288,5 +313,37 @@ QDebug operator<<(QDebug dbg, const MediaPlayer::State &s)
     return dbg.space();
 }
 
+void MediaPlayer::setAudioFade(qreal fade)
+{
+    m_fadeAmount = fade;
+    setVolumeInternal();
+}
+
+void MediaPlayer::setAudioVolume(int volume)
+{
+    m_volume = volume;
+    setVolumeInternal();
+}
+
+void MediaPlayer::setVolumeInternal()
+{
+    libvlc_audio_set_volume(m_player, m_volume * m_fadeAmount);
+}
+
+void MediaPlayer::setCdTrack(int track)
+{
+    libvlc_media_player_stop(m_player);
+    m_media->setCdTrack(track);
+    libvlc_media_player_set_media(m_player, *m_media);
+    libvlc_media_player_play(m_player);
+}
+
+#if (LIBVLC_VERSION_INT >= LIBVLC_VERSION(2, 2, 0, 0))
+void MediaPlayer::setEqualizer(libvlc_equalizer_t *equalizer)
+{
+    libvlc_media_player_set_equalizer(m_player, equalizer);
+}
+#endif
+
 } // namespace VLC
 } // namespace Phonon
