@@ -2,7 +2,7 @@
     Copyright (C) 2007-2008 Tanguy Krotoff <tkrotoff@gmail.com>
     Copyright (C) 2008 Lukas Durfina <lukas.durfina@gmail.com>
     Copyright (C) 2009 Fathi Boudra <fabo@kde.org>
-    Copyright (C) 2009-2011 vlc-phonon AUTHORS
+    Copyright (C) 2009-2011 vlc-phonon AUTHORS <kde-multimedia@kde.org>
     Copyright (C) 2011-2012 Harald Sitter <sitter@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -28,6 +28,7 @@
 
 #include "utils/debug.h"
 #include "mediaobject.h"
+#include "media.h"
 
 #include "video/videomemorystream.h"
 
@@ -204,10 +205,8 @@ VideoWidget::~VideoWidget()
         m_surfacePainter->widget = 0; // Lazy delete
 }
 
-void VideoWidget::connectToMediaObject(MediaObject *mediaObject)
+void VideoWidget::handleConnectToMediaObject(MediaObject *mediaObject)
 {
-    SinkNode::connectToMediaObject(mediaObject);
-
     connect(mediaObject, SIGNAL(hasVideoChanged(bool)),
             SLOT(updateVideoSize(bool)));
     connect(mediaObject, SIGNAL(hasVideoChanged(bool)),
@@ -218,17 +217,16 @@ void VideoWidget::connectToMediaObject(MediaObject *mediaObject)
     clearPendingAdjusts();
 }
 
-void VideoWidget::disconnectFromMediaObject(MediaObject *mediaObject)
+void VideoWidget::handleDisconnectFromMediaObject(MediaObject *mediaObject)
 {
-    SinkNode::disconnectFromMediaObject(mediaObject);
     // Undo all connections or path creation->destruction->creation can cause
     // duplicated connections or getting singals from two different MediaObjects.
     disconnect(mediaObject, 0, this, 0);
 }
 
-void VideoWidget::addToMedia(Media *media)
+void VideoWidget::handleAddToMedia(Media *media)
 {
-    SinkNode::addToMedia(media);
+    media->addOption(":video");
 
     if (!m_surfacePainter) {
 #if defined(Q_OS_MAC)
